#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Asterisk AGI Speaking Clock with Time Loop
Announcements every 15 seconds (0, 15, 30, 45), Beep before Text, Time shifted forward by 15 seconds
Standalone script with no external dependencies except gTTS and ffmpeg
Volker Lange-Janson, 9 March 2025
asterisk@janson-soft.de
Freely available for anyone on my website – do with it what you will!

Development:
https://grok.com/share/bGVnYWN5_a13004c1-937d-47ec-ae5f-12076ca83a8b
"""

import sys
import datetime
import time
import logging
import os
from gtts import gTTS

# Logging configuration
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(message)s')
logger = logging.getLogger(__name__)

# Constants
INTERVALS = [0, 15, 30, 45]  # Announcements at these seconds
TIME_SHIFT = 15  # Seconds by which the announced time is shifted forward

# AGI-specific functions
def init_env():
    """Initialises the AGI environment and returns it as a dictionary."""
    env = {}
    while True:
        line = sys.stdin.readline().strip()
        if line == '':
            break
        key, data = line.split(':', 1)
        if key[:4] != 'agi_':
            sys.stderr.write("Did not work!\n")
            sys.stderr.flush()
            continue
        key = key.strip()
        data = data.strip()
        if key != '':
            env[key] = data

    sys.stderr.write("AGI Environment Dump:\n")
    sys.stderr.flush()
    for key in env.keys():
        sys.stderr.write(f" -- {key} = {env[key]}\n")
        sys.stderr.flush()
    
    return env

def sayit(params):
    """Plays an audio file via Asterisk."""
    sys.stderr.write(f"STREAM FILE {str(params)} \"\"\n")
    sys.stderr.flush()
    sys.stdout.write(f"STREAM FILE {str(params)} \"\"\n")
    sys.stdout.flush()
    result = sys.stdin.readline().strip()
    return result

def text_to_speech(text, output_file="output"):
    """Converts text to speech and saves it as a WAV file."""
    try:
        output_file = f"/usr/share/asterisk/sounds/test/en/{output_file}"
        temp_mp3 = f"{output_file}.mp3"
        temp_wav = f"{output_file}.wav"
        tts = gTTS(text=text, lang="en", tld="co.uk")  # British English

        tts.save(temp_mp3)
        os.system(f"ffmpeg -i {temp_mp3} -ar 8000 -ac 1 -y {temp_wav}")
        os.remove(temp_mp3)

        sys.stderr.write(f"Text-to-Speech file saved as {temp_wav}\n")
        sys.stderr.flush()
        return temp_wav
    except Exception as e:
        sys.stderr.write(f"Error during text-to-speech conversion: {e}\n")
        sys.stderr.flush()
        return None

def play_text_as_audio(text, output_file="output"):
    """Converts text to speech, streams it to Asterisk, and deletes the WAV file."""
    wav_file = text_to_speech(text, output_file)
    if wav_file:
        sayit(wav_file.replace(".wav", ""))
        os.remove(wav_file)
        sys.stderr.write(f"Deleted WAV file: {wav_file}\n")
        sys.stderr.flush()

# Initialise AGI environment
agi_env = init_env()

# Clock-specific functions
def generate_time_str(tone_time):
    """Generates the time announcement text with time shifted forward, in British English."""
    shifted_time = tone_time + datetime.timedelta(seconds=TIME_SHIFT)
    
    hour = shifted_time.hour
    minute = shifted_time.minute
    second = shifted_time.second
    
    second_str = "1 second" if second == 1 else f"{second} seconds"
    minute_str = "1 minute" if minute == 1 else f"{minute} minutes"
    
    return f"At the next tone, it will be {hour} o'clock, {minute_str} and {second_str}"

def play_tone():
    """Plays the Asterisk internal beep tone."""
    sayit("beep")

def get_next_announcement_time():
    """Calculates the next announcement time in 15-second intervals."""
    now = datetime.datetime.now()
    current_second = now.second
    current_minute = now.replace(second=0, microsecond=0)
    
    for interval in INTERVALS:
        if interval > current_second or (interval == 0 and current_second > 45):
            next_time = current_minute.replace(second=interval)
            if interval == 0 and current_second > 45:
                next_time += datetime.timedelta(minutes=1)
            return next_time
    return current_minute + datetime.timedelta(minutes=1)

def announce_time(tone_time):
    """Plays the tone first, then the shifted time announcement."""
    play_tone()
    time_str = generate_time_str(tone_time)
    logger.info(f"Announcement: {time_str}")
    play_text_as_audio(time_str, output_file=f"time_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).hour}_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).minute}_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).second}")

def main():
    """Main function with time loop for the AGI script."""
    try:
        logger.info("AGI Speaking Clock with Time Loop started (Beep before Text, Time +15s)")
        sys.stderr.write("AGI Speaking Clock with Time Loop started (Beep before Text, Time +15s)\n")
        sys.stderr.flush()

        while True:
            next_time = get_next_announcement_time()
            now = datetime.datetime.now()
            wait_seconds = (next_time - now).total_seconds()

            if wait_seconds > 0:
                logger.info(f"Waiting {wait_seconds:.2f} seconds until {next_time.strftime('%H:%M:%S')}")
                time.sleep(wait_seconds)

            announce_time(next_time)
            time.sleep(0.5)

    except KeyboardInterrupt:
        logger.info("Programme terminated by user")
        sys.stderr.write("Programme terminated by user\n")
        sys.stderr.flush()
    except Exception as e:
        logger.error(f"Error in AGI script: {e}")
        sys.stderr.write(f"Error in AGI script: {e}\n")
        sys.stderr.flush()
        sys.exit(1)

if __name__ == "__main__":
    main()
