#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Asterisk AGI Talande Klocka med Tidsslinga (Speaking Clock with Time Loop)
Tillkännagivanden var 15:e sekund (0, 15, 30, 45), Beep före Text, Tid förskjuten framåt 15 sekunder
Självständigt skript utan externa beroenden förutom gTTS och ffmpeg
Volker Lange-Janson, 9 mars 2025
asterisk@janson-soft.de
Fritt tillgängligt för alla på min webbplats – gör vad du vill med det!

Utveckling:
https://grok.com/share/bGVnYWN5_a13004c1-937d-47ec-ae5f-12076ca83a8b
"""

import sys
import datetime
import time
import logging
import os
from gtts import gTTS

# Loggingkonfiguration
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(message)s')
logger = logging.getLogger(__name__)

# Konstanter
INTERVALS = [0, 15, 30, 45]  # Tillkännagivanden vid dessa sekunder
TIME_SHIFT = 15  # Sekunder som den annonserade tiden förskjuts framåt

# AGI-specifika funktioner
def init_env():
    """Initialiserar AGI-miljön och returnerar den som en ordbok."""
    env = {}
    while True:
        line = sys.stdin.readline().strip()
        if line == '':
            break
        key, data = line.split(':', 1)
        if key[:4] != 'agi_':
            sys.stderr.write("Fungerade inte!\n")
            sys.stderr.flush()
            continue
        key = key.strip()
        data = data.strip()
        if key != '':
            env[key] = data

    sys.stderr.write("AGI-miljödump:\n")
    sys.stderr.flush()
    for key in env.keys():
        sys.stderr.write(f" -- {key} = {env[key]}\n")
        sys.stderr.flush()
    
    return env

def sayit(params):
    """Spelar upp en ljudfil via Asterisk."""
    sys.stderr.write(f"STREAM FILE {str(params)} \"\"\n")
    sys.stderr.flush()
    sys.stdout.write(f"STREAM FILE {str(params)} \"\"\n")
    sys.stdout.flush()
    result = sys.stdin.readline().strip()
    return result

def text_to_speech(text, output_file="output"):
    """Omvandlar text till tal och sparar det som en WAV-fil."""
    try:
        output_file = f"/usr/share/asterisk/sounds/test/sv/{output_file}"
        temp_mp3 = f"{output_file}.mp3"
        temp_wav = f"{output_file}.wav"
        tts = gTTS(text=text, lang="sv")  # Svenska

        tts.save(temp_mp3)
        os.system(f"ffmpeg -i {temp_mp3} -ar 8000 -ac 1 -y {temp_wav}")
        os.remove(temp_mp3)

        sys.stderr.write(f"Text-till-tal-fil sparad som {temp_wav}\n")
        sys.stderr.flush()
        return temp_wav
    except Exception as e:
        sys.stderr.write(f"Fel vid text-till-tal-omvandling: {e}\n")
        sys.stderr.flush()
        return None

def play_text_as_audio(text, output_file="output"):
    """Omvandlar text till tal, strömmar det till Asterisk och tar bort WAV-filen."""
    wav_file = text_to_speech(text, output_file)
    if wav_file:
        sayit(wav_file.replace(".wav", ""))
        os.remove(wav_file)
        sys.stderr.write(f"Raderad WAV-fil: {wav_file}\n")
        sys.stderr.flush()

# Initialisera AGI-miljön
agi_env = init_env()

# Klockspecifika funktioner
def generate_time_str(tone_time):
    """Genererar tillkännagivandetexten med tiden förskjuten framåt, på svenska."""
    shifted_time = tone_time + datetime.timedelta(seconds=TIME_SHIFT)
    
    hour = shifted_time.hour
    minute = shifted_time.minute
    second = shifted_time.second
    
    second_str = "1 sekund" if second == 1 else f"{second} sekunder"
    minute_str = "1 minut" if minute == 1 else f"{minute} minuter"
    
    return f"Vid nästa ton är klockan {hour}, {minute_str} och {second_str}"

def play_tone():
    """Spelar upp Asterisks interna beep-ton."""
    sayit("beep")

def get_next_announcement_time():
    """Beräknar nästa tillkännagivandetid i 15-sekundersintervall."""
    now = datetime.datetime.now()
    current_second = now.second
    current_minute = now.replace(second=0, microsecond=0)
    
    for interval in INTERVALS:
        if interval > current_second or (interval == 0 and current_second > 45):
            next_time = current_minute.replace(second=interval)
            if interval == 0 and current_second > 45:
                next_time += datetime.timedelta(minutes=1)
            return next_time
    return current_minute + datetime.timedelta(minutes=1)

def announce_time(tone_time):
    """Spelar upp tonen först, sedan det förskjutna tillkännagivandet."""
    play_tone()
    time_str = generate_time_str(tone_time)
    logger.info(f"Tillkännagivande: {time_str}")
    play_text_as_audio(time_str, output_file=f"time_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).hour}_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).minute}_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).second}")

def main():
    """Huvudfunktion med tidsslinga för AGI-skriptet."""
    try:
        logger.info("AGI Talande Klocka med Tidsslinga startad (Beep före Text, Tid +15s)")
        sys.stderr.write("AGI Talande Klocka med Tidsslinga startad (Beep före Text, Tid +15s)\n")
        sys.stderr.flush()

        while True:
            next_time = get_next_announcement_time()
            now = datetime.datetime.now()
            wait_seconds = (next_time - now).total_seconds()

            if wait_seconds > 0:
                logger.info(f"Väntar {wait_seconds:.2f} sekunder till {next_time.strftime('%H:%M:%S')}")
                time.sleep(wait_seconds)

            announce_time(next_time)
            time.sleep(0.5)

    except KeyboardInterrupt:
        logger.info("Programmet avslutades av användaren")
        sys.stderr.write("Programmet avslutades av användaren\n")
        sys.stderr.flush()
    except Exception as e:
        logger.error(f"Fel i AGI-skriptet: {e}")
        sys.stderr.write(f"Fel i AGI-skriptet: {e}\n")
        sys.stderr.flush()
        sys.exit(1)

if __name__ == "__main__":
    main()
